@@ -3,11 +3,10 @@
 
 #include <linux/version.h>
 #include <linux/config.h>
+#include <linux/string.h>
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,0)
 #if defined(CONFIG_MODVERSIONS) && !defined(__GENKSYMS__) && !defined(__DEPEND__)
-#define MODVERSIONS
-#include <linux/modversions.h>
 #include "sndversions.h"
 #endif
 #endif
@@ -15,7 +14,11 @@
 #include <linux/kmod.h>
 #include <linux/devfs_fs_kernel.h>
 
-#include "../alsa-kernel/core/wrappers.c"
+/* defined in adriver.h but we don't include it... */
+#include <linux/compiler.h>
+#ifndef __nocast
+#define __nocast
+#endif
 
 #ifndef CONFIG_HAVE_STRLCPY
 #define strlcat snd_compat_strlcat
@@ -41,7 +44,7 @@ size_t snd_compat_strlcat(char *dest, const char *src, size_t count)
 }
 #endif
 
-#ifndef CONFIG_HAVE_SNPRINTF
+#ifndef CONFIG_HAVE_VSNPRINTF
 #define vsnprintf snd_compat_vsnprintf
 int snd_compat_vsnprintf(char *buf, size_t size, const char *fmt, va_list args);
 #endif
@@ -114,22 +117,6 @@ void snd_compat_request_module(const char *fmt, ...)
 }
 #endif
 
-#if LINUX_VERSION_CODE < KERNEL_VERSION(2, 3, 0)
-
-void snd_wrapper_request_region(unsigned long from, unsigned long extent, const char *name)
-{
-	return request_region(from, extent, name);
-}
-
-#ifdef CONFIG_OLD_KILL_FASYNC
-void snd_wrapper_kill_fasync(struct fasync_struct **fp, int sig, int band)
-{
-	kill_fasync(*(fp), sig);
-}
-#endif
-
-#endif /* < 2.3.0 */
-
 #if defined(CONFIG_DEVFS_FS)
 
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2, 5, 29)
@@ -215,7 +202,7 @@ int snd_pci_dev_present(const struct pci_device_id *ids)
 /*
  * msleep wrapper
  */
-#if (LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 0) && LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 6)) || (LINUX_VERSION_CODE < KERNEL_VERSION(2, 4, 29))
+#ifndef CONFIG_HAVE_MSLEEP
 #include <linux/delay.h>
 void snd_compat_msleep(unsigned int msecs)
 {
@@ -226,4 +213,18 @@ void snd_compat_msleep(unsigned int msecs)
 		timeout = schedule_timeout(timeout);
 	}
 }
+#endif
+
+#ifndef CONFIG_HAVE_MSLEEP_INTERRUPTIBLE
+#include <linux/delay.h>
+unsigned long snd_compat_msleep_interruptible(unsigned int msecs)
+{
+	unsigned long timeout = ((msecs) * HZ + 999) / 1000;
+
+	while (timeout && !signal_pending(current)) {
+		set_current_state(TASK_INTERRUPTIBLE);
+		timeout = schedule_timeout(timeout);
+	}
+	return (timeout * 1000) / HZ;
+}
 #endif /* < 2.6.6 */
