@@ -28,6 +28,7 @@
 #include <linux/slab.h>
 #include <linux/tty.h>
 #include <linux/moduleparam.h>
+#include <linux/platform_device.h>
 #include <asm/uaccess.h>
 #include <sound/core.h>
 #include <sound/rawmidi.h>
@@ -85,7 +86,7 @@ MODULE_PARM_DESC(handshake, "Do handshaking.");
 #define SERIAL_MODE_BIT_OUTPUT_TRIGGERED (3)
 
 typedef struct _snd_serialmidi {
-	snd_card_t *card;
+	struct snd_card *card;
 	char *sdev;			/* serial device name (e.g. /dev/ttyS0) */
 	int dev_idx;
 	unsigned int speed;		/* speed in bauds */
@@ -93,12 +94,12 @@ typedef struct _snd_serialmidi {
 	unsigned long mode;		/* see SERIAL_MODE_* */
 	unsigned int outs;		/* count of outputs */
 	unsigned char prev_status[SNDRV_SERIAL_MAX_OUTS];
-	snd_rawmidi_t *rmidi;		/* rawmidi device */
-	snd_rawmidi_substream_t *substream_input;
-	snd_rawmidi_substream_t *substream_output;
+	struct snd_rawmidi *rmidi;		/* rawmidi device */
+	struct snd_rawmidi_substream *substream_input;
+	struct snd_rawmidi_substream *substream_output;
 	struct file *file;
 	struct tty_struct *tty;
-	struct semaphore open_lock;
+	struct mutex open_lock;
 	void (*old_receive_buf)(struct tty_struct *, const unsigned char *cp, char *fp, int count);
 	void (*old_write_wakeup)(struct tty_struct *);
 	int old_exclusive;
@@ -107,7 +108,7 @@ typedef struct _snd_serialmidi {
 	unsigned char tx_buf[TX_BUF_SIZE];
 } serialmidi_t;
 
-static snd_card_t *snd_serialmidi_cards[SNDRV_CARDS] = SNDRV_DEFAULT_PTR;
+static struct platform_device *devptrs[SNDRV_CARDS];
 
 static void ldisc_receive_buf(struct tty_struct *, const unsigned char *cp, char *fp, int count);
 static void ldisc_write_wakeup(struct tty_struct *);
@@ -135,7 +136,7 @@ static int open_tty(serialmidi_t *serial, unsigned int mode)
 	struct tty_driver *driver;
 	int ldisc, speed, cflag;
 
-	down(&serial->open_lock);
+	mutex_lock(&serial->open_lock);
 	if (serial->tty) {
 		set_bit(mode, &serial->mode);
 		goto __end;
@@ -253,7 +254,7 @@ static int open_tty(serialmidi_t *serial, unsigned int mode)
       			serial->file = NULL;
       		}
       	}
-	up(&serial->open_lock);
+	mutex_unlock(&serial->open_lock);
 	return retval;
 }
 
@@ -263,7 +264,7 @@ static int close_tty(serialmidi_t *serial, unsigned int mode)
 			SERIAL_MODE_BIT_OUTPUT : SERIAL_MODE_BIT_INPUT;
 	struct tty_struct *tty;
 
-	down(&serial->open_lock);
+	mutex_lock(&serial->open_lock);
 	clear_bit(mode, &serial->mode);
 	if (test_bit(imode, &serial->mode))
 		goto __end;
@@ -282,7 +283,7 @@ static int close_tty(serialmidi_t *serial, unsigned int mode)
 	serial->tty = NULL;
 	serial->file = NULL;
       __end:
-	up(&serial->open_lock);
+	mutex_unlock(&serial->open_lock);
 	return 0;
 }
 
@@ -325,7 +326,7 @@ static void tx_loop(serialmidi_t *serial)
 		count = snd_rawmidi_transmit_peek(serial->substream_output, buf, count);
 		if (count > 0) {
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 10)
-			count = driver->write(tty, buf, count);
+			driver->write(tty, buf, count);
 #else
 			count = driver->write(tty, 0, buf, count);
 #endif
@@ -345,7 +346,7 @@ static void ldisc_write_wakeup(struct tty_struct *tty)
 	tx_loop(serial);
 }
 
-static void snd_serialmidi_output_trigger(snd_rawmidi_substream_t * substream, int up)
+static void snd_serialmidi_output_trigger(struct snd_rawmidi_substream *substream, int up)
 {
 	serialmidi_t *serial = substream->rmidi->private_data;
 	
@@ -359,7 +360,7 @@ static void snd_serialmidi_output_trigger(snd_rawmidi_substream_t * substream, i
 	}
 }
 
-static void snd_serialmidi_input_trigger(snd_rawmidi_substream_t * substream, int up)
+static void snd_serialmidi_input_trigger(struct snd_rawmidi_substream *substream, int up)
 {
 	serialmidi_t *serial = substream->rmidi->private_data;
 
@@ -370,7 +371,7 @@ static void snd_serialmidi_input_trigger(snd_rawmidi_substream_t * substream, in
 	}
 }
 
-static int snd_serialmidi_output_open(snd_rawmidi_substream_t * substream)
+static int snd_serialmidi_output_open(struct snd_rawmidi_substream *substream)
 {
 	serialmidi_t *serial = substream->rmidi->private_data;
 	int err;
@@ -381,7 +382,7 @@ static int snd_serialmidi_output_open(snd_rawmidi_substream_t * substream)
 	return 0;
 }
 
-static int snd_serialmidi_output_close(snd_rawmidi_substream_t * substream)
+static int snd_serialmidi_output_close(struct snd_rawmidi_substream *substream)
 {
 	serialmidi_t *serial = substream->rmidi->private_data;
 
@@ -389,7 +390,7 @@ static int snd_serialmidi_output_close(snd_rawmidi_substream_t * substream)
 	return close_tty(serial, SERIAL_MODE_BIT_OUTPUT);
 }
 
-static int snd_serialmidi_input_open(snd_rawmidi_substream_t * substream)
+static int snd_serialmidi_input_open(struct snd_rawmidi_substream *substream)
 {
 	serialmidi_t *serial = substream->rmidi->private_data;
 	int err;
@@ -400,7 +401,7 @@ static int snd_serialmidi_input_open(snd_rawmidi_substream_t * substream)
 	return 0;
 }
 
-static int snd_serialmidi_input_close(snd_rawmidi_substream_t * substream)
+static int snd_serialmidi_input_close(struct snd_rawmidi_substream *substream)
 {
 	serialmidi_t *serial = substream->rmidi->private_data;
 
@@ -408,14 +409,14 @@ static int snd_serialmidi_input_close(snd_rawmidi_substream_t * substream)
 	return close_tty(serial, SERIAL_MODE_BIT_INPUT);
 }
 
-static snd_rawmidi_ops_t snd_serialmidi_output =
+static struct snd_rawmidi_ops snd_serialmidi_output =
 {
 	.open =		snd_serialmidi_output_open,
 	.close =	snd_serialmidi_output_close,
 	.trigger =	snd_serialmidi_output_trigger,
 };
 
-static snd_rawmidi_ops_t snd_serialmidi_input =
+static struct snd_rawmidi_ops snd_serialmidi_input =
 {
 	.open =		snd_serialmidi_input_open,
 	.close =	snd_serialmidi_input_close,
@@ -430,7 +431,7 @@ static int snd_serialmidi_free(serialmidi_t *serial)
 	return 0;
 }
 
-static int snd_serialmidi_dev_free(snd_device_t *device)
+static int snd_serialmidi_dev_free(struct snd_device *device)
 {
 	serialmidi_t *serial = device->device_data;
 	return snd_serialmidi_free(serial);
@@ -438,7 +439,7 @@ static int snd_serialmidi_dev_free(snd_device_t *device)
 
 static int __init snd_serialmidi_rmidi(serialmidi_t *serial)
 {
-        snd_rawmidi_t *rrawmidi;
+        struct snd_rawmidi *rrawmidi;
         int err;
 
         if ((err = snd_rawmidi_new(serial->card, "UART Serial MIDI", serial->dev_idx, serial->outs, 1, &rrawmidi)) < 0)
@@ -454,12 +455,11 @@ static int __init snd_serialmidi_rmidi(serialmidi_t *serial)
         return 0;
 }
 
-static int __init snd_serialmidi_create(snd_card_t *card, const char *sdev,
+static int __init snd_serialmidi_create(struct snd_card *card, const char *sdev,
 					unsigned int speed, unsigned int adaptor,
-					unsigned int outs, int idx, int hshake,
-					serialmidi_t **rserial)
+					unsigned int outs, int idx, int hshake)
 {
-	static snd_device_ops_t ops = {
+	static struct snd_device_ops ops = {
 		.dev_free =	snd_serialmidi_dev_free,
 	};
 	serialmidi_t *serial;
@@ -482,16 +482,16 @@ static int __init snd_serialmidi_create(snd_card_t *card, const char *sdev,
 	}
 	if (outs < 1)
 		outs = 1;
-	if (outs > 16)
+	else if (outs > 16)
 		outs = 16;
 
-	if ((serial = kcalloc(1, sizeof(*serial), GFP_KERNEL)) == NULL)
+	if ((serial = kzalloc(sizeof(*serial), GFP_KERNEL)) == NULL)
 		return -ENOMEM;
 
-	init_MUTEX(&serial->open_lock);
+	mutex_init(&serial->open_lock);
 	serial->card = card;
 	serial->dev_idx = idx;
-	serial->sdev = snd_kmalloc_strdup(sdev, GFP_KERNEL);
+	serial->sdev = kstrdup(sdev, GFP_KERNEL);
 	if (serial->sdev == NULL) {
 		snd_serialmidi_free(serial);
 		return -ENOMEM;
@@ -513,16 +513,14 @@ static int __init snd_serialmidi_create(snd_card_t *card, const char *sdev,
 		return err;
 	}
 	
-	if (rserial)
-		*rserial = serial;
 	return 0;
 }
 
-static int __init snd_card_serialmidi_probe(int dev)
+static int __init snd_serial_probe(struct platform_device *devptr)
 {
-	snd_card_t *card;
-	serialmidi_t *serial;
+	struct snd_card *card;
 	int err;
+	int dev = devptr->id;
 
 	card = snd_card_new(index[dev], id[dev], THIS_MODULE, 0);
 	if (card == NULL)
@@ -562,7 +560,7 @@ static int __init snd_card_serialmidi_probe(int dev)
 			sprintf(devname2, "%s%d", devname, start_dev + i);
 			if ((err = snd_serialmidi_create(card, devname2, speed[dev],
 							 adaptor[dev], outs[dev], i,
-							 handshake[dev], &serial)) < 0) {
+							 handshake[dev])) < 0) {
 				snd_card_free(card);
 				return err;
 			}
@@ -570,46 +568,89 @@ static int __init snd_card_serialmidi_probe(int dev)
 	} else {
 		if ((err = snd_serialmidi_create(card, sdev[dev], speed[dev],
 						 adaptor[dev], outs[dev], 0,
-						 handshake[dev], &serial)) < 0) {
+						 handshake[dev])) < 0) {
 			snd_card_free(card);
 			return err;
 		}
 	}
 
 	sprintf(card->longname, "%s at %s", card->shortname, sdev[dev]);
+
+	snd_card_set_dev(card, &devptr->dev);
+
 	if ((err = snd_card_register(card)) < 0) {
 		snd_card_free(card);
 		return err;
 	}
-	snd_serialmidi_cards[dev] = card;
+
+	platform_set_drvdata(devptr, card);
 	return 0;
 }
 
+static int snd_serial_remove(struct platform_device *devptr)
+{
+	snd_card_free(platform_get_drvdata(devptr));
+	platform_set_drvdata(devptr, NULL);
+	return 0;
+}
+
+#define SND_SERIAL_DRIVER	"snd_serial"
+
+static struct platform_driver snd_serial_driver = {
+	.probe		= snd_serial_probe,
+	.remove		= snd_serial_remove,
+	.driver		= {
+		.name	= SND_SERIAL_DRIVER
+	},
+};
+
+static void __init_or_module snd_serial_unregister_all(void)
+{
+	int i;
+
+	for (i = 0; i < ARRAY_SIZE(devices); ++i)
+		platform_device_unregister(devptrs[i]);
+	platform_driver_unregister(&snd_serial_driver);
+}
+
 static int __init alsa_card_serialmidi_init(void)
 {
-	int dev, cards = 0;
+	int i, cards, err;
 
-	for (dev = 0; dev < SNDRV_CARDS; dev++) {
-		if (!enable[dev])
+	if ((err = platform_driver_register(&snd_serial_driver)) < 0)
+		return err;
+
+	cards = 0;
+	for (i = 0; i < SNDRV_CARDS; i++) {
+		struct platform_device *device;
+		if (! enable[i])
 			continue;
-		if (snd_card_serialmidi_probe(dev) >= 0)
-			cards++;
+		device = platform_device_register_simple(SND_SERIAL_DRIVER,
+							 i, NULL, 0);
+		if (IS_ERR(device)) {
+			err = PTR_ERR(device);
+			goto errout;
+		}
+		devptrs[i] = device;
+		cards++;
 	}
 	if (!cards) {
 #ifdef MODULE
 		printk(KERN_ERR "serial MIDI device not found or device busy\n");
 #endif
-		return -ENODEV;
+		err = -ENODEV;
+		goto errout;
 	}
 	return 0;
+
+ errout:
+	snd_serial_unregister_all();
+	return err;
 }
 
 static void __exit alsa_card_serialmidi_exit(void)
 {
-	int idx;
-
-	for (idx = 0; idx < SNDRV_CARDS; idx++)
-		snd_card_free(snd_serialmidi_cards[idx]);
+	snd_serial_unregister_all();
 }
 
 module_init(alsa_card_serialmidi_init)
