@@ -33,7 +33,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * msnd.c,v 1.3 2004/03/06 16:55:13 tiwai Exp
+ * msnd.c,v 1.4 2005/01/31 16:05:51 cladisch Exp
  *
  ********************************************************************/
 #define __NO_VERSION__
@@ -47,7 +47,11 @@
 #include <linux/vmalloc.h>
 #include <linux/types.h>
 #include <linux/delay.h>
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 10) /* enable_irq(), disable_irq() */
+#include <linux/interrupt.h>
+#else
 #include <asm/irq.h>
+#endif
 #include <asm/io.h>
 #include "msnd.h"
 
