@@ -28,22 +28,20 @@
 // ****************************************************************************
 
 
-static int set_input_clock(echoaudio_t *chip, u16 clock);
-static int set_professional_spdif(echoaudio_t *chip, char prof);
-static int set_nominal_level(echoaudio_t *chip, u16 index, char consumer);
-static int update_flags(echoaudio_t *chip);
-static int set_vmixer_gain(echoaudio_t *chip, u16 output, u16 pipe, int gain);
-static int update_vmixer_level(echoaudio_t *chip);
+static int set_input_clock(struct echoaudio *chip, u16 clock);
+static int set_professional_spdif(struct echoaudio *chip, char prof);
+static int update_flags(struct echoaudio *chip);
+static int set_vmixer_gain(struct echoaudio *chip, u16 output, u16 pipe, int gain);
+static int update_vmixer_level(struct echoaudio *chip);
 
 
-static int init_hw(echoaudio_t *chip, u16 device_id, u16 subdevice_id)
+static int init_hw(struct echoaudio *chip, u16 device_id, u16 subdevice_id)
 {
-	int err, i;
+	int err;
 
 	DE_INIT(("init_hw() - Mia\n"));
 	snd_assert((subdevice_id & 0xfff0) == MIA, return -ENODEV);
 
-	/* This part is common to all the cards */
 	if ((err = init_dsp_comm_page(chip))) {
 		DE_INIT(("init_hw - could not initialize DSP comm page\n"));
 		return err;
@@ -59,23 +57,13 @@ static int init_hw(echoaudio_t *chip, u16 device_id, u16 subdevice_id)
 		chip->has_midi = TRUE;
 	chip->input_clock_types = ECHO_CLOCK_BIT_INTERNAL | ECHO_CLOCK_BIT_SPDIF;
 
-	/* Load the DSP and the ASIC on the PCI card */
 	if ((err = load_firmware(chip)) < 0)
 		return err;
-
 	chip->bad_board = FALSE;
 
-	/* Must call this here after DSP is init to init gains and mutes */
 	if ((err = init_line_levels(chip)))
 		return err;
 
-	/* Set professional nominal levels (FALSE is +4dBu) */
-	for (i = 0; i < num_analog_busses_out(chip); i++)
-		err = set_nominal_level(chip, i, FALSE);
-
-	for (i = 0; i < num_analog_busses_in(chip); i++)
-		err = set_nominal_level(chip, bx_analog_in(chip) + i, FALSE);
-
 	/* Default routing of the virtual channels: vchannels 0-3 go to analog
 	outputs and vchannels 4-7 go to S/PDIF outputs */
 	set_vmixer_gain(chip, 0, 0, 0);
@@ -94,20 +82,7 @@ static int init_hw(echoaudio_t *chip, u16 device_id, u16 subdevice_id)
 
 
 
-//===========================================================================
-//
-// detect_input_clocks returns a bitmask consisting of all the input
-// clocks currently connected to the hardware; this changes as the user
-// connects and disconnects clock inputs.
-//
-// You should use this information to determine which clocks the user is
-// allowed to select.
-//
-// Mia supports S/PDIF input clock.
-//
-//===========================================================================
-
-static u32 detect_input_clocks(const echoaudio_t *chip)
+static u32 detect_input_clocks(const struct echoaudio *chip)
 {
 	u32 clocks_from_dsp, clock_bits;
 
@@ -125,28 +100,14 @@ static u32 detect_input_clocks(const echoaudio_t *chip)
 
 
 /* The Mia has no ASIC. Just do nothing */
-static int load_asic(echoaudio_t *chip)
+static int load_asic(struct echoaudio *chip)
 {
 	return 0;
 }
 
 
 
-/****************************************************************************
-
-	Hardware setup and config
-
- ****************************************************************************/
-
-//===========================================================================
-//
-// set_sample_rate
-//
-// Set the audio sample rate for Mia
-//
-//===========================================================================
-
-static int set_sample_rate(echoaudio_t *chip, u32 rate)
+static int set_sample_rate(struct echoaudio *chip, u32 rate)
 {
 	u32 control_reg;
 
@@ -180,11 +141,10 @@ static int set_sample_rate(echoaudio_t *chip, u32 rate)
 		if (wait_handshake(chip))
 			return -EIO;
 
-		/* Set the values in the comm page */
 		chip->comm_page->sample_rate = cpu_to_le32(rate);	/* ignored by the DSP */
 		chip->comm_page->control_register = cpu_to_le32(control_reg);
 		chip->sample_rate = rate;
-		/* Poke the DSP */
+
 		clear_handshake(chip);
 		return send_vector(chip, DSP_VC_UPDATE_CLOCKS);
 	}
@@ -193,13 +153,7 @@ static int set_sample_rate(echoaudio_t *chip, u32 rate)
 
 
 
-//===========================================================================
-//
-//      Set the input clock
-//
-//===========================================================================
-
-static int set_input_clock(echoaudio_t *chip, u16 clock)
+static int set_input_clock(struct echoaudio *chip, u16 clock)
 {
 	DE_ACT(("set_input_clock(%d)\n", clock));
 	snd_assert(clock == ECHO_CLOCK_INTERNAL || clock == ECHO_CLOCK_SPDIF, return -EINVAL);
@@ -211,7 +165,7 @@ static int set_input_clock(echoaudio_t *chip, u16 clock)
 
 
 /* This function routes the sound from a virtual channel to a real output */
-static int set_vmixer_gain(echoaudio_t *chip, u16 output, u16 pipe, int gain)
+static int set_vmixer_gain(struct echoaudio *chip, u16 output, u16 pipe, int gain)
 {
 	int index;
 
@@ -231,7 +185,7 @@ static int set_vmixer_gain(echoaudio_t *chip, u16 output, u16 pipe, int gain)
 
 
 /* Tell the DSP to read and update virtual mixer levels in comm page. */
-static int update_vmixer_level(echoaudio_t *chip)
+static int update_vmixer_level(struct echoaudio *chip)
 {
 	if (wait_handshake(chip))
 		return -EIO;
@@ -242,7 +196,7 @@ static int update_vmixer_level(echoaudio_t *chip)
 
 
 /* Tell the DSP to reread the flags from the comm page */
-static int update_flags(echoaudio_t *chip)
+static int update_flags(struct echoaudio *chip)
 {
 	if (wait_handshake(chip))
 		return -EIO;
@@ -252,7 +206,7 @@ static int update_flags(echoaudio_t *chip)
 
 
 
-static int set_professional_spdif(echoaudio_t *chip, char prof)
+static int set_professional_spdif(struct echoaudio *chip, char prof)
 {
 	DE_ACT(("set_professional_spdif %d\n", prof));
 	if (prof)
